==> flow-0.50.0.tar.gz: flow-0.50.0/src/services/inference/infer_service.ml <==
(**
 * Copyright (c) 2013-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the "flow" directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 *)

open Utils_js

let apply_docblock_overrides (metadata: Context.metadata) docblock_info =
  let open Context in

  let local_metadata = metadata.local_metadata in

  let local_metadata = { local_metadata with jsx = Docblock.jsx docblock_info } in

  let local_metadata = match Docblock.flow docblock_info with
  | None -> local_metadata
  | Some Docblock.OptIn -> { local_metadata with checked = true; }
  | Some Docblock.OptInWeak -> { local_metadata with checked = true; weak = true }

  (* --all (which sets metadata.checked = true) overrides @noflow, so there are
     currently no scenarios where we'd change checked = true to false. in the
     future, there may be a case where checked defaults to true (but is not
     forced to be true ala --all), but for now we do *not* want to force
     checked = false here. *)
  | Some Docblock.OptOut -> local_metadata
  in

  let local_metadata = match Docblock.preventMunge docblock_info with
  | Some value -> { local_metadata with munge_underscores = not value; }
  | None -> local_metadata
  in

  { metadata with local_metadata }

(* Given a filename, retrieve the parsed AST, derive a module name,
   and invoke the local (infer) pass. This will build and return a
   fresh context object for the module. *)
let infer_module ~(metadata: Context.metadata) ~lint_settings filename =
  let ast = Parsing_service_js.get_ast_unsafe filename in
  let info = Parsing_service_js.get_docblock_unsafe filename in
  let metadata = apply_docblock_overrides metadata info in
  let require_loc_map = Parsing_service_js.get_requires_unsafe filename in
  Type_inference_js.infer_ast ~metadata ~filename ~lint_settings ast ~require_loc_map

(* local inference job:
   takes list of filenames, accumulates into parallel lists of
   filenames, error sets *)
let infer_job ~options acc files =
  let metadata = Context.metadata_of_options options in
  let lint_settings = Some options.Options.opt_lint_settings in
  List.fold_left (fun acc file ->
    let file_str = string_of_filename file in
    try Profile_utils.checktime ~options ~limit:1.0
      ~msg:(fun t -> spf "perf: inferred %s in %f" file_str t)
      ~log:(fun t -> Flow_server_profile.infer ~filename:file_str ~infer_time:t)
      ~f:(fun () ->
        (* prerr_endlinef "[%d] INFER: %s" (Unix.getpid()) file_str; *)

        (* infer produces a context for this module *)
        let cx = infer_module ~metadata ~lint_settings file in
        (* note: save and clear errors and error suppressions before storing
         * cx to shared heap *)
        let errs = Context.errors cx in
        let suppressions = Context.error_suppressions cx in
        let lint_settings = Context.lint_settings cx in
        Context.remove_all_errors cx;
        Context.remove_all_error_suppressions cx;
        Context.remove_all_lint_settings cx;

        Context_cache.add ~audit:Expensive.ok cx;

        (* add filename, errorset, suppressions *)
        (Context.file cx, errs, suppressions, lint_settings) :: acc
      )
    with
    (* Unrecoverable exceptions *)
    | SharedMem_js.Out_of_shared_memory
    | SharedMem_js.Heap_full
    | SharedMem_js.Hash_table_full
    | SharedMem_js.Dep_table_full as exc -> raise exc
    (* A catch all suppression is probably a bad idea... *)
    | exc ->
      let loc = Loc.({ none with source = Some file }) in
      let msg = Flow_error.(EInternal (loc, InferJobException exc)) in
      let error = Flow_error.error_of_msg ~trace_reasons:[] ~op:None ~source_file:file msg in
      let errorset = Errors.ErrorSet.singleton error in
      prerr_endlinef "(%d) infer_job THROWS: %s"
        (Unix.getpid()) (fmt_file_exc (string_of_filename file) exc);
      (file, errorset, Error_suppressions.empty, SuppressionMap.invalid_default) :: acc
  ) acc files

(* local type inference pass.
   Returns a set of successfully inferred files.
   Creates contexts for inferred files, with errors in cx.errors *)
let infer ~options ~workers files =
  Profile_utils.logtime ~options
    ~msg:(fun t -> spf "inferred %d files in %f" (List.length files) t)
    ~f:(fun () ->
      MultiWorker.call
        workers
        ~job: (infer_job ~options)
        ~neutral: []
        ~merge: List.rev_append
        ~next: (MultiWorker.next workers files)
    )

==> flow-0.52.0.tar.gz: flow-0.52.0/src/services/inference/infer_service.ml <==
(**
 * Copyright (c) 2013-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the "flow" directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 *)

open Utils_js

let apply_docblock_overrides (metadata: Context.metadata) docblock_info =
  let open Context in

  let local_metadata = metadata.local_metadata in

  let local_metadata = { local_metadata with jsx = Docblock.jsx docblock_info } in

  let local_metadata = match Docblock.flow docblock_info with
  | None -> local_metadata
  | Some Docblock.OptIn -> { local_metadata with checked = true; }
  | Some Docblock.OptInWeak -> { local_metadata with checked = true; weak = true }

  (* --all (which sets metadata.checked = true) overrides @noflow, so there are
     currently no scenarios where we'd change checked = true to false. in the
     future, there may be a case where checked defaults to true (but is not
     forced to be true ala --all), but for now we do *not* want to force
     checked = false here. *)
  | Some Docblock.OptOut -> local_metadata
  in

  let local_metadata = match Docblock.preventMunge docblock_info with
  | Some value -> { local_metadata with munge_underscores = not value; }
  | None -> local_metadata
  in

  { metadata with local_metadata }

(* Given a filename, retrieve the parsed AST, derive a module name,
   and invoke the local (infer) pass. This will build and return a
   fresh context object for the module. *)
let infer_module ~(metadata: Context.metadata) ~lint_settings filename =
  let ast = Parsing_service_js.get_ast_unsafe filename in
  let info = Parsing_service_js.get_docblock_unsafe filename in
  let metadata = apply_docblock_overrides metadata info in
  let require_loc_map = Parsing_service_js.get_requires_unsafe filename in
  Type_inference_js.infer_ast ~metadata ~filename ~lint_settings ast ~require_loc_map

(* local inference job:
   takes list of filenames, accumulates into parallel lists of
   filenames, error sets *)
let infer_job ~options acc files =
  let metadata = Context.metadata_of_options options in
  let lint_settings = Some options.Options.opt_lint_settings in
  List.fold_left (fun acc file ->
    let file_str = string_of_filename file in
    try Profile_utils.checktime ~options ~limit:1.0
      ~msg:(fun t -> spf "perf: inferred %s in %f" file_str t)
      ~log:(fun t -> Flow_server_profile.infer ~filename:file_str ~infer_time:t)
      ~f:(fun () ->
        (* prerr_endlinef "[%d] INFER: %s" (Unix.getpid()) file_str; *)

        (* infer produces a context for this module *)
        let cx = infer_module ~metadata ~lint_settings file in
        (* note: save and clear errors and error suppressions before storing
         * cx to shared heap *)
        let errs = Context.errors cx in
        let suppressions = Context.error_suppressions cx in
        let lint_settings = Context.lint_settings cx in
        Context.remove_all_errors cx;
        Context.remove_all_error_suppressions cx;
        Context.remove_all_lint_settings cx;

        Context_cache.add ~audit:Expensive.ok cx;

        (* add filename, errorset, suppressions *)
        (Context.file cx, errs, suppressions, lint_settings) :: acc
      )
    with
    (* Unrecoverable exceptions *)
    | SharedMem_js.Out_of_shared_memory
    | SharedMem_js.Heap_full
    | SharedMem_js.Hash_table_full
    | SharedMem_js.Dep_table_full as exc -> raise exc
    (* A catch all suppression is probably a bad idea... *)
    | exc ->
      let loc = Loc.({ none with source = Some file }) in
      let msg = Flow_error.(EInternal (loc, InferJobException exc)) in
      let error = Flow_error.error_of_msg ~trace_reasons:[] ~op:None ~source_file:file msg in
      let errorset = Errors.ErrorSet.singleton error in
      prerr_endlinef "(%d) infer_job THROWS: %s"
        (Unix.getpid()) (fmt_file_exc (string_of_filename file) exc);
      (file, errorset, Error_suppressions.empty, LintSettingsMap.invalid_default) :: acc
  ) acc files

(* local type inference pass.
   Returns a set of successfully inferred files.
   Creates contexts for inferred files, with errors in cx.errors *)
let infer ~options ~workers files =
  Profile_utils.logtime ~options
    ~msg:(fun t -> spf "inferred %d files in %f" (List.length files) t)
    ~f:(fun () ->
      MultiWorker.call
        workers
        ~job: (infer_job ~options)
        ~neutral: []
        ~merge: List.rev_append
        ~next: (MultiWorker.next workers files)
    )
